@@ -17,18 +17,14 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-#if HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "libbluray/bdnav/meta_data.h"
-#include "libbluray/bluray.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <inttypes.h>
 #include <unistd.h>
 
+#include "libbluray/bdnav/meta_data.h"
+#include "libbluray/bluray.h"
+
 static const char *_yes_no(int i)
 {
     return i > 0 ? "yes" : i < 0 ? "unknown" : "no";
@@ -186,6 +182,8 @@ int main(int argc, char *argv[])
                 printf("Java VM found       : %s\n", _yes_no(info->libjvm_detected));
                 printf("BD-J handled        : %s\n", _yes_no(info->bdj_handled));
             }
+            printf("BD-J organization ID: %s\n", info->bdj_org_id);
+            printf("BD-J disc ID        : %s\n", info->bdj_disc_id);
         }
     }
 
