@@ -62,7 +62,7 @@ static int _dir_read_posix(BD_DIR_H *dir, BD_DIRENT *entry)
 
 static BD_DIR_H *_dir_open_posix(const char* dirname)
 {
-    BD_DIR_H *dir = malloc(sizeof(BD_DIR_H));
+    BD_DIR_H *dir = calloc(1, sizeof(BD_DIR_H));
 
     BD_DEBUG(DBG_DIR, "Opening POSIX dir %s... (%p)\n", dirname, (void*)dir);
     dir->close = _dir_close_posix;
