@@ -43,24 +43,36 @@ int win32_mkdir(const char *dir)
     return _wmkdir(wdir);
 }
 
-const char *file_get_config_home(void)
+char *win32_get_font_dir(const char *font_file)
+{
+    wchar_t wdir[MAX_PATH];
+    if (S_OK != SHGetFolderPathW(NULL, CSIDL_FONTS, NULL, SHGFP_TYPE_CURRENT, wdir)) {
+        GetWindowsDirectoryW(wdir, MAX_PATH);
+        wcscat(wdir, L"\\fonts");
+    }
+
+    int   len  = WideCharToMultiByte (CP_UTF8, 0, wdir, -1, NULL, 0, NULL, NULL);
+    char *path = malloc(len + strlen(font_file) + 2);
+    WideCharToMultiByte(CP_UTF8, 0, wdir, -1, path, len, NULL, NULL);
+    path[len - 1] = '\\';
+    strcpy(path + len, font_file);
+    return path;
+}
+
+char *file_get_config_home(void)
 {
     return file_get_data_home();
 }
 
-const char *file_get_data_home(void)
+char *file_get_data_home(void)
 {
-    static char *appdir = NULL;
     wchar_t wdir[MAX_PATH];
 
-    if (appdir)
-        return appdir;
-
     /* Get the "Application Data" folder for the user */
     if (S_OK == SHGetFolderPathW(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE,
                                  NULL, SHGFP_TYPE_CURRENT, wdir)) {
         int len = WideCharToMultiByte (CP_UTF8, 0, wdir, -1, NULL, 0, NULL, NULL);
-        appdir = malloc(len);
+        char *appdir = malloc(len);
         WideCharToMultiByte (CP_UTF8, 0, wdir, -1, appdir, len, NULL, NULL);
         return appdir;
     }
@@ -69,7 +81,7 @@ const char *file_get_data_home(void)
     return NULL;
 }
 
-const char *file_get_cache_home(void)
+char *file_get_cache_home(void)
 {
     return file_get_data_home();
 }
