@@ -71,18 +71,18 @@ int libbdplus_required(const char *device_path)
     BD_FILE_H *fd;
     char      *tmp;
 
-    tmp = str_printf("%s/BDSVM/00000.svm", device_path);
+    tmp = str_printf("%s" DIR_SEP "BDSVM" DIR_SEP "00000.svm", device_path);
     fd = file_open(tmp, "rb");
     X_FREE(tmp);
 
     if (fd) {
         file_close(fd);
 
-        BD_DEBUG(DBG_BLURAY, "BDSVM/00000.svm found. Disc seems to be BD+ protected.\n");
+        BD_DEBUG(DBG_BLURAY, "BDSVM" DIR_SEP "00000.svm found. Disc seems to be BD+ protected.\n");
         return 1;
     }
 
-    BD_DEBUG(DBG_BLURAY, "BDSVM/00000.svm not found. No BD+ protection.\n");
+    BD_DEBUG(DBG_BLURAY, "BDSVM" DIR_SEP "00000.svm not found. No BD+ protection.\n");
     return 0;
 }
 
