@@ -42,23 +42,19 @@ typedef enum {
     BDJ_EVENT_RATE,
 } BDJ_EVENT;
 
-/* bdj_get_uo_mask() */
-#define BDJ_MENU_CALL_MASK     0x01
-#define BDJ_TITLE_SEARCH_MASK  0x02
+typedef struct {
+    char *persistent_root;
+    char *cache_root;
+} BDJ_STORAGE;
 
 typedef struct bdjava_s BDJAVA;
 
 struct bluray;
-struct bd_argb_buffer_s;
-
-typedef void (*bdj_overlay_cb)(struct bluray *, const unsigned *, int, int,
-                               int, int, int, int);
 
 BD_PRIVATE BDJAVA* bdj_open(const char *path, struct bluray *bd,
-                            bdj_overlay_cb osd_cb, struct bd_argb_buffer_s *buf);
+                            const char *bdj_disc_id, BDJ_STORAGE *storage);
 BD_PRIVATE void bdj_close(BDJAVA *bdjava);
 BD_PRIVATE int  bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param);
-BD_PRIVATE int  bdj_get_uo_mask(BDJAVA *bdjava);
 
 BD_PRIVATE int  bdj_jvm_available(void); /* 0: no. 1: only jvm. 2: jvm + libbluray.jar. */
 
