@@ -18,9 +18,8 @@
 
 package java.io;
 
-import java.net.URL;
-//import java.nio.channels.FileChannel;
 
+import org.videolan.BDJLoader;
 import org.videolan.BDJXletContext;
 import org.videolan.Logger;
 
@@ -50,11 +49,28 @@ public class FileInputStream extends InputStream
         fd.incrementAndGetUseCount();
 
         if (file.isAbsolute()) {
+            String cachedName = BDJLoader.getCachedFile(name);
+            if (cachedName != name) {
+                if (logger == null) {
+                    logger = Logger.getLogger(FileInputStream.class.getName());
+                }
+                logger.info("Using cached " + cachedName + " for " + name);
+                name = cachedName;
+            }
             open(name);
         } else {
             /* relative paths are problematic ... */
             /* Those should be mapped to xlet home directory, which is inside .jar file. */
-            openResource(name);
+
+            String home = BDJXletContext.getCurrentXletHome();
+            if (home == null) {
+                if (logger == null) {
+                    logger = Logger.getLogger(FileInputStream.class.getName());
+                }
+                logger.error("no home found for " + name + " at " + Logger.dumpStack());
+                throw new FileNotFoundException(name);
+            }
+            open(home + name);
         }
 
         available = 1024;
@@ -77,43 +93,6 @@ public class FileInputStream extends InputStream
         available = 1024;
     }
 
-    private void openResource(String name) throws FileNotFoundException {
-        if (logger == null) {
-            logger = Logger.getLogger(FileInputStream.class.getName());
-        }
-
-        /* J2SE URL.getPath() returns file:/xxx.jar!/... for jar resource urls ... */
-        if (name.indexOf("file:/") == 0 && name.indexOf(".jar!/") > 0) {
-            logger.error("Fixing invalid resource url: " + name);
-            name = name.substring(name.indexOf(".jar!") + 6);
-        }
-
-        /* try to locate file in Xlet home directory (inside JAR file) */
-
-        ClassLoader cldr = (ClassLoader)BDJXletContext.getCurrentClassLoader();
-        if (cldr == null) {
-            throw new FileNotFoundException(name);
-        }
-
-        if (name.startsWith("./") || name.startsWith(".\\")) {
-            name = name.substring(2);
-        }
-
-        URL url = cldr.getResource(name);
-        if (url == null) {
-            logger.error("Resource not found: " + name);
-            throw new FileNotFoundException(name);
-        }
-        logger.info(name + " translated to " + url);
-
-        fd.slave = cldr.getResourceAsStream(name);
-        if (fd.slave == null) {
-            logger.error("Error getting resource as stream");
-            throw new FileNotFoundException(name);
-        }
-    }
-
-
     private native void open(String name) throws FileNotFoundException;
     private native int  readBytes(byte b[], int off, int len) throws IOException;
     private native int  close0();
@@ -211,7 +190,7 @@ public class FileInputStream extends InputStream
 
     protected void finalize() throws IOException {
         if (fd != null) {
-            if (fd != fd.in) {
+            if (fd != FileDescriptor.in) {
                 close(false);
             }
         }
