@@ -21,7 +21,6 @@
 package org.videolan.media.content.playlist;
 
 import java.awt.Component;
-import java.util.LinkedList;
 
 import org.bluray.media.UOMaskTableControl;
 import org.bluray.media.UOMaskTableListener;
@@ -34,7 +33,6 @@ import org.videolan.PlaylistInfo;
 
 public class UOMaskTableControlImpl implements UOMaskTableControl {
     protected UOMaskTableControlImpl(Handler player) {
-        this.player = player;
     }
 
     public Component getControlComponent() {
@@ -72,5 +70,4 @@ public class UOMaskTableControlImpl implements UOMaskTableControl {
     }
 
     private BDJListeners listeners = new BDJListeners();
-    private Handler player;
 }
