@@ -30,7 +30,7 @@ typedef struct bdid_s {
 } BDID_DATA;
 
 
-BD_PRIVATE BDID_DATA* bdid_parse(const char *path); /* parse id.bdmv */
+BD_PRIVATE BDID_DATA* bdid_parse(const char *disc_root); /* parse id.bdmv */
 BD_PRIVATE void       bdid_free(BDID_DATA **p);
 
 #endif // _BDID_PARSE_H_
