@@ -51,10 +51,11 @@ _human_readable_sig(char *sig, uint32_t s1, uint32_t s2)
 static int
 _parse_stream_attr(BITSTREAM *bits, CLPI_PROG_STREAM *ss)
 {
-    int len, pos;
+    int64_t pos;
+    int len;
 
     if (!bs_is_align(bits, 0x07)) {
-        fprintf(stderr, "_parse_stream_attr: Stream alignment error\n");
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_stream_attr(): Stream alignment error\n");
     }
 
     len = bs_read(bits, 8);
@@ -89,22 +90,22 @@ _parse_stream_attr(BITSTREAM *bits, CLPI_PROG_STREAM *ss)
         case 0xa2:
             ss->format = bs_read(bits, 4);
             ss->rate   = bs_read(bits, 4);
-            bs_read_bytes(bits, ss->lang, 3);
+            bs_read_string(bits, ss->lang, 3);
             break;
 
         case 0x90:
         case 0x91:
         case 0xa0:
-            bs_read_bytes(bits, ss->lang, 3);
+            bs_read_string(bits, ss->lang, 3);
             break;
 
         case 0x92:
             ss->char_code = bs_read(bits, 8);
-            bs_read_bytes(bits, ss->lang, 3);
+            bs_read_string(bits, ss->lang, 3);
             break;
 
         default:
-            fprintf(stderr, "stream attr: unrecognized coding type %02x\n", ss->coding_type);
+            BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_stream_attr(): unrecognized coding type %02x\n", ss->coding_type);
             break;
     };
     ss->lang[3] = '\0';
@@ -129,7 +130,7 @@ _parse_header(BITSTREAM *bits, CLPI_CL *cl)
 
         _human_readable_sig(sig, cl->type_indicator, cl->type_indicator2);
         _human_readable_sig(expect, CLPI_SIG1, CLPI_SIG2A);
-        fprintf(stderr, "failed signature match expected (%s) got (%s)\n", 
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "failed signature match expected (%s) got (%s)\n",
                 expect, sig);
         return 0;
     }
@@ -144,7 +145,8 @@ _parse_header(BITSTREAM *bits, CLPI_CL *cl)
 static int
 _parse_clipinfo(BITSTREAM *bits, CLPI_CL *cl)
 {
-    int len, pos;
+    int64_t pos;
+    int len;
     int ii;
 
     bs_seek_byte(bits, 40);
@@ -168,8 +170,7 @@ _parse_clipinfo(BITSTREAM *bits, CLPI_CL *cl)
     pos = bs_pos(bits) >> 3;
     if (len) {
         cl->clip.ts_type_info.validity = bs_read(bits, 8);
-        bs_read_bytes(bits, cl->clip.ts_type_info.format_id, 4);
-        cl->clip.ts_type_info.format_id[4] = '\0';
+        bs_read_string(bits, cl->clip.ts_type_info.format_id, 4);
         // Seek past the stuff we don't know anything about
         bs_seek_byte(bits, pos + len);
     }
@@ -181,10 +182,8 @@ _parse_clipinfo(BITSTREAM *bits, CLPI_CL *cl)
             malloc(cl->clip.atc_delta_count * sizeof(CLPI_ATC_DELTA));
         for (ii = 0; ii < cl->clip.atc_delta_count; ii++) {
             cl->clip.atc_delta[ii].delta = bs_read(bits, 32);
-            bs_read_bytes(bits, cl->clip.atc_delta[ii].file_id, 5);
-            cl->clip.atc_delta[ii].file_id[5] = '\0';
-            bs_read_bytes(bits, cl->clip.atc_delta[ii].file_code, 4);
-            cl->clip.atc_delta[ii].file_code[4] = '\0';
+            bs_read_string(bits, cl->clip.atc_delta[ii].file_id, 5);
+            bs_read_string(bits, cl->clip.atc_delta[ii].file_code, 4);
             bs_skip(bits, 8);
         }
     }
@@ -196,8 +195,7 @@ _parse_clipinfo(BITSTREAM *bits, CLPI_CL *cl)
         if (cl->font_info.font_count) {
             cl->font_info.font = malloc(cl->font_info.font_count * sizeof(CLPI_FONT));
             for (ii = 0; ii < cl->font_info.font_count; ii++) {
-                bs_read_bytes(bits, cl->font_info.font[ii].file_id, 5);
-                cl->font_info.font[ii].file_id[5] = '\0';
+                bs_read_string(bits, cl->font_info.font[ii].file_id, 5);
                 bs_skip(bits, 8);
             }
         }
@@ -324,7 +322,7 @@ _parse_cpi(BITSTREAM *bits, CLPI_CPI *cpi)
 
     bs_skip(bits, 12);
     cpi->type = bs_read(bits, 4);
-    ep_map_pos = bs_pos(bits) >> 3;
+    ep_map_pos = (uint32_t)(bs_pos(bits) >> 3);
 
     // EP Map starts here
     bs_skip(bits, 8);
@@ -689,7 +687,7 @@ _clpi_parse(const char *path)
 
     fp = file_open(path, "rb");
     if (fp == NULL) {
-        fprintf(stderr, "Failed to open %s\n", path);
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "Failed to open %s\n", path);
         X_FREE(cl);
         return NULL;
     }
@@ -743,7 +741,7 @@ clpi_parse(const char *path)
         char *backup = malloc(len + 8);
 
         strncpy(backup, path, len - 18);
-        strcpy(backup + len - 18, "BACKUP/");
+        strcpy(backup + len - 18, "BACKUP" DIR_SEP);
         strcpy(backup + len - 18 + 7, path + len - 18);
 
         cl = _clpi_parse(backup);
