@@ -20,7 +20,7 @@
 #if !defined(_HDMV_VM_H_)
 #define _HDMV_VM_H_
 
-#include <util/attributes.h>
+#include "util/attributes.h"
 
 #include <stdint.h>
 
@@ -65,7 +65,6 @@ typedef struct hdmv_vm_event_s {
  */
 
 struct bd_registers_s;
-struct indx_root_s;
 
 /*
  *
@@ -73,10 +72,11 @@ struct indx_root_s;
 
 typedef struct hdmv_vm_s HDMV_VM;
 
-BD_PRIVATE HDMV_VM *hdmv_vm_init(const char *disc_root, struct bd_registers_s *regs, struct indx_root_s *indx);
+BD_PRIVATE HDMV_VM *hdmv_vm_init(const char *disc_root, struct bd_registers_s *regs,
+                                 unsigned num_titles, unsigned first_play_available, unsigned top_menu_available);
 BD_PRIVATE void     hdmv_vm_free(HDMV_VM **p);
 
-BD_PRIVATE int      hdmv_vm_select_object(HDMV_VM *p, int object);
+BD_PRIVATE int      hdmv_vm_select_object(HDMV_VM *p, uint32_t object);
 BD_PRIVATE int      hdmv_vm_set_object(HDMV_VM *p, int num_nav_cmds, void *nav_cmds);
 BD_PRIVATE int      hdmv_vm_run(HDMV_VM *p, HDMV_EVENT *ev);
 BD_PRIVATE int      hdmv_vm_get_event(HDMV_VM *p, HDMV_EVENT *ev);
